function [bX, bZ, seX, seZ, stats] = EWreg(y, x, z, meth, bXint, SE)
% EWreg - linear regression with one mismeasured regressor, using the
%         Erickson Whited [2000 JPE] method
%
% y     - dependant variable - a [N,1] vector
%
% x     - independant mismeasured regressor - a [N,1] vector
%
% z     - independant perfectly measured regressors - a [N,NZ] matrix
%         note that Z must include an intercept (a vecotr of ones)
%
% meth  - the method to use, one of {'GMM3', 'GMM4', 'GMM5', 'GMM6', 'GMM7'}
%       - Default: 'GMM3'
%
% bXint - initial values to use in GMM optimization for bX (only if meth is not 'GMM3')
%         a [M,1] vector of initial values.
%         This can be a list of values, and all values are ok except zero.
%         If a zero is found in the list, it will be replaced by the
%         defualt value. So it is advisable to have zero always be a member
%         of the list. The initial value yielding the lowest objective function 
%         will be used.
%       - Default: E(y_d^2*x_d)/E(y_d*x_d^2)
%
% SE    - Should we account for SE as if Ex and Ey are known?
%       - This will disable the SE adjustment when finding the optimal
%         weighting matrix for the GMM step
%       - Default: 0 (No)
%
%
% Output:
% bX    - scalar coefficient on the mismeasured regressor
%
% bZ    - [NZ,1] coefficients on the perfectly measured regressors
%
% seX   - scalar standard error on the mismeasured regressor coeff
%
% seZ   - [NZ,1] SE on the perfectly measured regressors coeff
%
% stats - a structure containing all statistics, including:
%       - bX      - scalar coefficient on the mismeasured regressor
%       - bZ      - [NZ,1] coefficients on the perfectly measured regressors
%       - seX     - scalar standard error on the mismeasured regressor coeff
%       - seZ     - [NZ,1] SE on the perfectly measured regressors coeff
%       - inflncX - influence function for mismeasured regressor
%       - inflncZ - influence functions for perfectly measured regressors
%       - Jstat   - the chi^2 J statistic
%       - Jval    - the p-value of the J statistic
%       - rho     - the R^2 value for the "true" regression
%       - tau     - the R^2 value for the mismeasured regression
%       - phirho  - influence function for rho^2
%       - phitau  - influence function for tau^2
%       - SErho   - standard error for rho^2
%       - SEtau   - standard error for tau^2
%       - w       - the weighting matrix used in GMM
%       - obj     - the value of the GMM objective function
%       - dfree   - degrees of freedom used in J test
%       - ID3val  - Newey West identification test value using 3rd moment
%       - ID3stat - identification test chi^2 statistic (3rd moment)
%       - ID4val  - Newey West identification test value using 3rd+4th moment
%       - ID4stat - identification test chi^2 statistic (3rd+4th moment)


if nargin<3 
    error('EWreg:nargin', 'Not enough arguments, review function documentation')
end

n =size(y,1);
nz=size(z,2);

if nz==0 || sum(sum(z==1)==n)==0
    error('EWreg:const', 'Z must contain a "1" column')
end

if nargin==3
    meth='GMM3';
    bXint=0;
    SE=0;
elseif nargin==4
    bXint=0;
    SE=0;
elseif nargin==5
    SE=0;
end

warning('off', 'MATLAB:nearlySingularMatrix');

mux=(z'*z)\(z'*x);
muy=(z'*z)\(z'*y);
y_d=y-z*muy;
x_d=x-z*mux;

y2_d=y_d.^2;
yx_d=(x_d.*y_d);
x2_d=x_d.^2;
y2x_d=(x_d.*y2_d);
yx2_d=(x2_d.*y_d);
y3x_d=y2x_d.*y_d;
y2x2_d=y2_d.*x2_d;
yx3_d=yx2_d.*x_d;

x3_d=x_d.*x2_d;
y3_d=y_d.*y2_d;
y4x_d=y3_d.*yx_d;
y3x2_d=y3_d.*x2_d;
y2x3_d=y2_d.*x3_d;
yx4_d=yx_d.*x3_d;
y4_d=y2_d.^2;
x4_d=x2_d.^2;
y5x_d=y_d.*y4x_d;
y4x2_d=y_d.*y3x2_d;
y3x3_d=y_d.*y2x3_d;
y2x4_d=y_d.*yx4_d;
yx5_d=yx4_d.*x_d;

y5_d=y2_d.*y3_d;
x5_d=x2_d.*x3_d;
y6x_d=y5_d.*yx_d;
y5x2_d=y5_d.*x2_d;
y4x3_d=y4_d.*x3_d;
y3x4_d=y3_d.*x4_d;
y2x5_d=y2_d.*x5_d;
yx6_d=yx_d.*x5_d;

mom={y2_d yx_d x2_d y2x_d yx2_d y3x_d y2x2_d yx3_d x3_d y3_d y4x_d ...
    y3x2_d y2x3_d yx4_d y4_d x4_d y5x_d y4x2_d y3x3_d y2x4_d yx5_d ...
    y5_d x5_d y6x_d y5x2_d y4x3_d y3x4_d y2x5_d yx6_d};


ey2_d=mean(y2_d);
eyx_d=mean(yx_d);
ex2_d=mean(x2_d);
ey2x_d=mean(y2x_d);
eyx2_d=mean(yx2_d);
ey3x_d=mean(y3x_d);
ey2x2_d=mean(y2x2_d);
eyx3_d=mean(yx3_d);

ex3_d=mean(x3_d);
ey3_d=mean(y3_d);
ey4x_d=mean(y4x_d);
ey3x2_d=mean(y3x2_d);
ey2x3_d=mean(y2x3_d);
eyx4_d=mean(yx4_d);
ey4_d=mean(y4_d);
ex4_d=mean(x4_d);
ey5x_d=mean(y5x_d);
ey4x2_d=mean(y4x2_d);
ey3x3_d=mean(y3x3_d);
ey2x4_d=mean(y2x4_d);
eyx5_d=mean(yx5_d);

ey5_d=mean(y5_d);
ex5_d=mean(x5_d);
ey6x_d=mean(y6x_d);
ey5x2_d=mean(y5x2_d);
ey4x3_d=mean(y4x3_d);
ey3x4_d=mean(y3x4_d);
ey2x5_d=mean(y2x5_d);
eyx6_d=mean(yx6_d);

emom={ey2_d eyx_d ex2_d ey2x_d eyx2_d ey3x_d ey2x2_d eyx3_d ex3_d ...
    ey3_d ey4x_d ey3x2_d ey2x3_d eyx4_d ey4_d ex4_d ey5x_d ey4x2_d ...
    ey3x3_d ey2x4_d eyx5_d ey5_d ex5_d ey6x_d ey5x2_d ey4x3_d ...
    ey3x4_d ey2x5_d eyx6_d};

% Make the moments for the correct standard errors for beta.
y6_d=y_d.^6;
x6_d=x_d.^6;

idx = ones(nz,1);
ey2_z   = mean(y2_d(:,idx).*z)';
eyx_z   = mean(yx_d(:,idx).*z)';
ex2_z   = mean(x2_d(:,idx).*z)';
ey2x_z  = mean(y2x_d(:,idx).*z)';
eyx2_z  = mean(yx2_d(:,idx).*z)';
ey3x_z  = mean(y3x_d(:,idx).*z)';
ey2x2_z = mean(y2x2_d(:,idx).*z)';
eyx3_z  = mean(yx3_d(:,idx).*z)';
ey3_z   = mean(y3_d(:,idx).*z)';
ex3_z   = mean(x3_d(:,idx).*z)';
ey4x_z  = mean(y4x_d(:,idx).*z)';
ey3x2_z = mean(y3x2_d(:,idx).*z)';
ey2x3_z = mean(y2x3_d(:,idx).*z)';
eyx4_z  = mean(yx4_d(:,idx).*z)';
ey4_z   = mean(y4_d(:,idx).*z)';
ex4_z   = mean(x4_d(:,idx).*z)';
ey5x_z  = mean(y5x_d(:,idx).*z)';
ey4x2_z = mean(y4x2_d(:,idx).*z)';
ey3x3_z = mean(y3x3_d(:,idx).*z)';
ey2x4_z = mean(y2x4_d(:,idx).*z)';
eyx5_z  = mean(yx5_d(:,idx).*z)';
ey5_z   = mean(y5_d(:,idx).*z)';
ex5_z   = mean(x5_d(:,idx).*z)';
ey6_z   = mean(y6_d(:,idx).*z)';
ex6_z   = mean(x6_d(:,idx).*z)';

ezz = (z'*z)/n;
inezz = inv(ezz);
zy_d = z.*y_d(:,idx);
zx_d = z.*x_d(:,idx);

zmom = {inezz zy_d zx_d eyx_z ey2_z ex2_z ey3_z ey2x_z eyx2_z ex3_z ey4_z ...
ey3x_z ey2x2_z eyx3_z ex4_z ey5_z ey4x_z ey3x2_z ey2x3_z eyx4_z ex5_z ...
ey6_z ey5x_z ey4x2_z ey3x3_z ey2x4_z eyx5_z ex6_z};


%----------here we do identification tests.------------------
for ID=3:4
    c   = EWIDcvec(emom,ID);
    neq = size(c,1);
    ff  = EWIDoptw(mom,zmom,n,neq,c,ID);
    w   = inv((ff'*ff)./n);
    c   = EWgmm(w,c,emom,nan,neq,ID);
    ff  = EWIDoptw(mom,zmom,n,neq,c,ID);
    w   = inv((ff'*ff)./n);
    if (ID==3)
        stats.ID3val  = n*c'*w*c;     %Newey West Wald test for c==0
        stats.ID3stat = 1-chi2cdf(stats.ID3val,2);
    elseif (ID==4)
        Cum4 = zeros(5,1);
        Cum4(1) = ey3x_d  - 3*eyx_d*ey2_d;
        Cum4(2) = ey2x2_d - ey2_d*ex2_d - 2*eyx_d^2;
        Cum4(3) = eyx3_d  - 3*eyx_d*ex2_d;
        Cum4(4) = ey2x_d;
        Cum4(5) = eyx2_d;
        
        Vee = zeros(5,neq);
        Vee(1,1) = -3*eyx_d;
        Vee(1,2) = -3*ey2_d;
        Vee(1,4) = 1;
        Vee(2,1) = -ex2_d;
        Vee(2,2) = -4*eyx_d;
        Vee(2,3) = -ey2_d;
        Vee(2,5) = 1;
        Vee(3,2) = -3*ex2_d;
        Vee(3,3) = -3*eyx_d;
        Vee(3,6) = 1;
        Vee(4,7) = 1;
        Vee(5,8) = 1;

        w = inv(Vee*((ff'*ff)./n)*Vee');
        stats.ID4val  = n*Cum4'*w*Cum4;
        stats.ID4stat = 1-chi2cdf(stats.ID4val,5);
    end
end

%-------------start the GMM part--------------------

if strcmp(meth,'GMM3')
    nc=5;
    neq=5;
    estim=1;
elseif strcmp(meth,'GMM4')
    nc=6;
    neq=8;
    estim=2;
elseif strcmp(meth,'GMM5')
    nc=9;
    neq=14;
    estim=3;
elseif strcmp(meth,'GMM6')
    nc=12;
    neq=21;
    estim=4;
elseif strcmp(meth,'GMM7')
    nc=15;
    neq=29;
    estim=5;
else
    error('EWreg:MethChk', 'Wrong method provided. Use GMMX, X={3..7}')
end

%----------here we input the starting values.----------------
if estim==1
    bXint = 0;
end
c       = zeros(nc,1);
bXlen   = size(bXint,1);
objsave = zeros(bXlen,1);

for rep=1:(bXlen+1)
    
    if rep<(bXlen+1)
        c(1,1) = bXint(rep);
    else
        if bXlen>1
            % restore best inital value
            [~,loc] = min(objsave);
            c(1,1) = bXint(loc);
        else
            % no point in restoring - only one value tested
            break
        end
    end
    
    if c(1,1)==0
        c(1,1)=ey2x_d/eyx2_d;
    end
    
    c(2,1)=eyx_d/c(1,1);
    c(3,1)=ey2_d-(c(1,1).^2)*c(2,1);
    c(4,1)=ex2_d-c(2,1);
    c(5,1)=eyx2_d/c(1,1);

    if estim>1
        c(6,1)=(eyx3_d-3*c(1,1)*c(2,1)*c(4,1))/c(1,1);
    end

    if estim>2
        c(7,1)=ex3_d-c(5,1);
        c(8,1)=ey3_d-(c(1,1).^3)*c(5,1);
        c(9,1)=(eyx4_d/c(1,1))-6*c(5,1)*c(4,1)-4*c(2,1)*c(7,1);
    end

    if estim>3
        c(10,1)=ey4_d-(c(1,1).^4)*c(6,1)-6*(c(1,1).^2)*c(2,1)*c(3,1);
        c(11,1)=ex4_d-c(6,1)-6*c(2,1)*c(4,1);
        c(12,1)=(eyx5_d/c(1,1))-10*c(6,1)*c(4,1)-10*c(5,1)*c(7,1)...
            -5*c(2,1)*c(11,1);
    end

    if estim>4
        c(13,1)=ey5_d-(c(1,1).^5)*c(9,1)-10*(c(1,1).^3)*c(5,1)*c(3,1)...
            -10*(c(1,1).^2)*c(2,1)*c(8,1);
        c(14,1)=ex5_d-c(9,1)-10*c(5,1)*c(4,1)-10*c(2,1)*c(7,1);
        c(15,1)=(eyx6_d/c(1,1))-15*c(9,1)*c(4,1)-20*c(6,1)*c(7,1)...
            -15*c(5,1)*c(11,1)-6*c(2,1)*c(14,1);
    end

    % We only need to maximize if the estimator is not exactly identified -> not
    % for GMM3, but for all the rest. Note that gradX calculates analytical
    % derivetives
    if estim>1
        %--------the program creates the weighting matrix.------------
        ff=EWoptw(mom,emom,zmom,n,neq,estim,SE);
        w=inv((ff'*ff)./n);
        
        c = EWgmm(w,c,emom,estim,neq,0);

    end; %----------end of exactly identified estimator condition------------

    % compute t-ratios,etc.

    ff  = EWoptw(mom,emom,zmom,n,neq,estim,SE);
    f   = EWdeff(c,emom,estim,neq);
    w   = inv((ff'*ff)./n);
    obj = f'*w*f;

    if rep<(bXlen+1)
        objsave(rep)=obj;
    end
end

stats.w     = w;
stats.obj   = obj;
stats.dfree = neq-nc;

if estim==1
    g=grad1(c,neq);
elseif estim==2
    g=grad2(c,neq);
elseif estim==3
    g=grad3(c,neq);
elseif estim==4
    g=grad4(c,neq);
elseif estim==5
    g=grad5(c,neq);
end

gwg=g'*w*g;
vc=inv(gwg)/n;
stderr=sqrt(diag(vc));
inflnc=-n*vc*g'*w*ff';

bX  = c(1,1);
seX = stderr(1,1);
stats.bX      = bX;
stats.seX     = seX;
stats.inflncX = inflnc(1,:)';

if estim>1
    stats.Jstat = obj*n;
    stats.Jval  =  1-chi2cdf(obj*n,neq-nc);
else
    stats.Jstat = nan;
    stats.Jval  = nan;
end

% Make bZ, seZ and friends

c11=c(1,1);
bZ = muy(:,1)-c11*mux(:,1);

gee=zeros((nz*2+1),nz);
gee(1:nz,1:nz)=eye(nz);
gee(nz+1:nz*2,1:nz)=-c11*eye(nz);
gee((nz*2+1),:)=-mux(:,1)';

bigphi = [([(inezz*zy_d'); (inezz*zx_d')]); (-inflnc(1,:))];
avar = (bigphi*bigphi')./n.^2;
seZ = sqrt(diag(gee'*avar*gee));
stats.bZ      = bZ;
stats.seZ     = seZ;
stats.inflncZ = -bigphi'*gee;


%--------------rho2 and tau2--------------------------------------------------
ez = mean(z);
sigz=(z-kron(ones(n,1),ez))'*(z-kron(ones(n,1),ez))/n;

numer=(muy'*sigz*muy + c(1,1).^2*c(2,1));
stats.rho=numer/(numer+c(3,1));

numer=(mux'*sigz*mux + c(2,1));
stats.tau=numer/(numer+c(4,1));

vecsigz=zeros(nz+nz*(nz-1)/2,1);
vecsigz(1:nz,1)=diag(sigz);

phiz=zeros(n,nz+nz*(nz-1)/2);
phiz(:,1:nz)=(z-kron(ones(n,1),ez)).*(z-kron(ones(n,1),ez));

% here we pack the top half of the sigz matrix into a vector
counter=nz+nz*(nz-1)/2;
for qq=nz:(-1):2
    for ee=(qq-1):(-1):1
        vecsigz(counter,1)=sigz(ee,qq);
        phiz(:,counter)=(z(:,ee)-kron(ones(n,1),ez(1,ee))).*(z(:,qq)-kron(ones(n,1),ez(1,qq)));
        counter=counter-1;
    end
end

phiz=phiz-kron(ones(n,1),vecsigz');
phimuy=inezz*zy_d';
phimux=inezz*zx_d';

%--make the influence functions for the standard errors for rho2 and tau2.
bigphi=[phimux ; phimuy ; (phiz') ; (-inflnc(1:4,:)) ];
avar=(bigphi*bigphi')./(n.^2);
%--------------first column is for rho2 and the second is for tau2.
gee = zeros(2*nz+(nz+nz*(nz-1)/2)+4,2);

%-----------first,do rho2---------------------------
numer=muy'*sigz*muy+c(1,1).^2*c(2,1);
denom=numer+c(3,1);
%------------derivatives wrt muy---------------------
for qq=1:nz
    gee(nz+qq,1)= (2*muy'*sigz(:,qq))/denom - numer*(2*muy'*sigz(:,qq))/(denom.^2);
end
%------------derivatives wrt the first part of sigz--
for qq=1:nz
    gee(2*nz+qq,1)=(muy(qq,1).^2)/denom-numer/(denom.^2)*muy(qq,1).^2;
end;
%------------derivatives wrt the second part of sigz--
counter=2*nz+(nz+nz*(nz-1)/2);
for qq=nz:(-1):2
    for ee=(qq-1):(-1):1
        gee(counter,1)= 2*muy(ee,1)*muy(qq,1)/denom-2*numer/(denom.^2)*muy(ee,1)*muy(qq,1);
        counter=counter-1;
    end
end
%------------derivatives wrt c--
counter=2*nz+(nz+nz*(nz-1)/2);
gee(counter+1,1)=2*c(1,1)*c(2,1)/denom-2*numer/(denom.^2)*c(1,1)*c(2,1);
gee(counter+2,1)=(c(1,1).^2)/denom-numer/(denom.^2)*c(1,1).^2;
gee(counter+3,1)=-numer/(denom.^2);

    
%-----------now for tau2------------------------
numer=mux'*sigz*mux+c(2,1);
denom=numer+c(4,1);
%------------derivatives wrt mux---------------------
for qq=1:nz
    gee(qq,2)=(2*mux'*sigz(:,qq))/denom-numer*(2*mux'*sigz(:,qq))/(denom.^2);
end
%------------derivatives wrt the first part of sigz--
for qq=1:nz
    gee(2*nz+qq,2)=(mux(qq,1).^2)/denom-numer/(denom.^2)*mux(qq,1).^2;
end
%------------derivatives wrt the second part of sigz--
counter=2*nz+(nz+nz*(nz-1)/2);
for qq=nz:(-1):2
    for ee=(qq-1):(-1):1
        gee(counter,2)=2*mux(ee,1)*mux(qq,1)/denom-2*numer/(denom.^2)*mux(ee,1)*mux(qq,1);
        counter=counter-1;
    end
end
%------------derivatives wrt c--
counter=2*nz+(nz+nz*(nz-1)/2);
gee(counter+2,2)=1/denom-numer/(denom.^2);    
gee(counter+4,2)=-numer/(denom.^2);

%--------------done with influnence funcs. Now use them.----------------------
vcrhotau=gee'*avar*gee;

stats.phirho=-bigphi'*gee(:,1);
stats.phitau=-bigphi'*gee(:,2);

stats.SErho=sqrt(vcrhotau(1,1));
stats.SEtau=sqrt(vcrhotau(2,2));

warning('on', 'MATLAB:nearlySingularMatrix');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Helper Functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% subroutine to define the c vector - the GMM coeff for identification test
function c = EWIDcvec(emom,ID)
if (ID==3)
    c = zeros(2,1);
    c(1,1) = emom{4};   %Ey2x
    c(2,1) = emom{5};   %Eyx2
elseif (ID==4)
    c = zeros(8,1);
    c(1,1) = emom{1};   %Ey2
    c(2,1) = emom{3};   %Ex2
    c(3,1) = emom{2};   %Eyx
    c(4,1) = emom{6};   %Ey3x
    c(5,1) = emom{7};   %Ey2x2
    c(6,1) = emom{8};   %Eyx3
    c(7,1) = emom{4};   %Ey2x
    c(8,1) = emom{5};   %Eyx2
else
    error('EWIDcvec:ID', 'Internal error - wrong ID value found');
end

%subroutine to define the f vector for the optimal weighting matrix.
function f = EWIDoptw(mom,zmom,n,neq,c,ID)
f=zeros(n,neq);

y2   = mom{1};
yx   = mom{2};
x2   = mom{3};
y2x  = mom{4};
yx2  = mom{5};
y3x  = mom{6};
y2x2 = mom{7};
yx3  = mom{8};

inezz = zmom{1};
zy_d  = zmom{2};
zx_d  = zmom{3};

eyx_z  = zmom{4};
ey2_z  = zmom{5};
ex2_z  = zmom{6};
ey3_z  = zmom{7};
ey2x_z = zmom{8};
eyx2_z = zmom{9};
ex3_z  = zmom{10};


if (ID==3)
    f(:,1) = y2x  - c(1,1) + (-2*eyx_z'*inezz*zy_d'  - ey2_z'*inezz*zx_d')';
    f(:,2) = yx2  - c(2,1) + (-ex2_z'*inezz*zy_d'    - 2*eyx_z'*inezz*zx_d')';
elseif (ID==4)
    f(:,1) = y2   - c(1,1);
    f(:,2) = yx   - c(2,1);
    f(:,3) = x2   - c(3,1);
    f(:,4) = y3x  - c(4,1) + (-3*ey2x_z'*inezz*zy_d' - ey3_z'*inezz*zx_d')';
    f(:,5) = y2x2 - c(5,1) + (-2*eyx2_z'*inezz*zy_d' - 2*ey2x_z'*inezz*zx_d')';
    f(:,6) = yx3  - c(6,1) + (-ex3_z'*inezz*zy_d'    - 3*eyx2_z'*inezz*zx_d')';
    f(:,7) = y2x  - c(7,1) + (-2*eyx_z'*inezz*zy_d'  - ey2_z'*inezz*zx_d')';
    f(:,8) = yx2  - c(8,1) + (-ex2_z'*inezz*zy_d'    - 2*eyx_z'*inezz*zx_d')';
else
    error('EWIDoptw:ID', 'Internal error - wrong ID value found');
end

% subroutine to run a GMM optimization
function c = EWgmm(w,c,emom,estim,neq,ID)
%-------------set internal optimization variables-------------
maxiter=299;
maxsqez=240;
tol=1e-9;

%-------------start of iteration loop------------------------
iter=1;
dc=1;                      % Initialize the step length.
while norm(dc,inf)>=tol;
    if (ID==3 || ID==4)
        f = EWIDcvec(emom,ID) - c;
        g = -eye(neq);
    else
        f=EWdeff(c,emom,estim,neq);          % The program jumps to the subroutine that
                                             % defines the f vector.
        if estim==2
            g=grad2(c,neq);                 % The program jumps to the subroutine that computes analytic derivatives.
        elseif estim==3                     % g is matrix of partials of f wrt params
            g=grad3(c,neq);
        elseif estim==4
            g=grad4(c,neq);
        elseif estim==5
            g=grad5(c,neq);
        end
    end

    obj = f'*w*f;               % This computes the value of the GMM objective function.

    gwg = g'*w*g;               % This uses the GAUSS-NEWTON method to compute full step dc.
    gwf = g'*w*f;
    dc  = gwg\gwf;

    if maxsqez > 0;               
        c_new=EWsqueez(c,dc,emom,maxsqez,obj,estim,neq,w,ID); % Adjust step length
    else
        c_new=c-dc;
    end;

    dc=c_new-c;                      % Update variables for the next iteration.
    c=c_new;

    iter=iter+1;
    if iter >= maxiter;                      % Quit iterating if necessary.
        break;
    end;

end; %------end of iteration loop------------------------------------


% subroutine to define the f vector - the GMM objective
function f = EWdeff(a,mmm,estim,neq)

f=zeros(1,neq);

f(:,1)=mmm{1}-(a(1,1).^2)*a(2,1)-a(3,1);
f(:,2)=mmm{2}-a(1,1)*a(2,1);
f(:,3)=mmm{3}-a(2,1)-a(4,1);
f(:,4)=mmm{4}-(a(1,1).^2)*a(5,1);
f(:,5)=mmm{5}-a(1,1)*a(5,1);

if estim>1
    f(:,6) =mmm{6}-(a(1,1).^3)*a(6,1)-3*a(1,1)*a(2,1)*a(3,1);
    f(:,7) =mmm{7}-(a(1,1).^2)*(a(6,1)+a(2,1)*a(4,1))...
            -a(3,1)*(a(2,1)+a(4,1));
    f(:,8) =mmm{8}-a(1,1)*a(6,1)-3*a(1,1)*a(2,1)*a(4,1);
end

if estim>2
    f(:,9) =mmm{9}-a(5,1)-a(7,1);
    f(:,10)=mmm{10}-(a(1,1).^3)*a(5,1)-a(8,1);
    f(:,11)=mmm{11}-(a(1,1).^4)*a(9,1)-6*(a(1,1).^2)*a(5,1)*a(3,1)...
            -4*a(1,1)*a(2,1)*a(8,1);
    f(:,12)=mmm{12}-(a(1,1).^3)*(a(9,1)+a(5,1)*a(4,1))...
            -3*a(1,1)*a(5,1)*a(3,1)-a(8,1)*(a(2,1)+a(4,1));
    f(:,13)=mmm{13}-(a(1,1).^2)*(a(9,1)+3*a(5,1)*a(4,1)+a(2,1)*a(7,1))...
            -a(3,1)*(a(5,1)+a(7,1));
    f(:,14)=mmm{14}-a(1,1)*(a(9,1)+6*a(5,1)*a(4,1)+4*a(2,1)*a(7,1));
end

if estim>3
    f(:,15)=mmm{15}-(a(1,1).^4)*a(6,1)-6*(a(1,1).^2)*a(2,1)*a(3,1)...
            -a(10,1);
    f(:,16)=mmm{16}-a(6,1)-6*a(2,1)*a(4,1)-a(11,1);
    f(:,17)=mmm{17}-(a(1,1).^5)*a(12,1)-10*(a(1,1).^3)*a(6,1)*a(3,1)...
            -10*(a(1,1).^2)*a(5,1)*a(8,1)-5*a(1,1)*a(2,1)*a(10,1);
    f(:,18)=mmm{18}-(a(1,1).^4)*(a(12,1)+a(6,1)*a(4,1))...
            -6*(a(1,1).^2)*(a(6,1)*a(3,1)+a(2,1)*a(3,1)*a(4,1))...
            -4*a(1,1)*a(5,1)*a(8,1)-a(10,1)*(a(2,1)+a(4,1));
    f(:,19)=mmm{19}-(a(1,1).^3)*(a(12,1)+3*a(6,1)*a(4,1)+a(5,1)*a(7,1))...
            -a(1,1)*(3*a(6,1)*a(3,1)+9*a(2,1)*a(3,1)*a(4,1))...
            -a(8,1)*(a(5,1)+a(7,1));
    f(:,20)=mmm{20}-(a(1,1).^2)*(a(12,1)+6*a(6,1)*a(4,1)...
            +4*a(5,1)*a(7,1)+a(2,1)*a(11,1))...
            -a(3,1)*(a(6,1)+6*a(2,1)*a(4,1)+a(11,1));
    f(:,21)=mmm{21}-a(1,1)*(a(12,1)+10*a(6,1)*a(4,1)...
            +10*a(5,1)*a(7,1)+5*a(2,1)*a(11,1));
end

if estim>4
    f(:,22)=mmm{22}-(a(1,1).^5)*a(9,1)-10*(a(1,1).^3)*a(5,1)*a(3,1)...
            -10*(a(1,1).^2)*a(2,1)*a(8,1)-a(13,1);
    f(:,23)=mmm{23}-a(9,1)-10*a(5,1)*a(4,1)-10*a(2,1)*a(7,1)-a(14,1);
    f(:,24)=mmm{24}-(a(1,1).^6)*a(15,1)-15*(a(1,1).^4)*a(9,1)*a(3,1)...
            -20*(a(1,1).^3)*a(6,1)*a(8,1)-15*(a(1,1).^2)*a(5,1)*a(10,1)...
            -6*a(1,1)*a(2,1)*a(13,1);
    f(:,25)=mmm{25}-(a(1,1).^5)*(a(15,1)+a(9,1)*a(4,1))...
            -10*(a(1,1).^3)*(a(9,1)*a(3,1)+a(5,1)*a(3,1)*a(4,1))...
            -10*(a(1,1).^2)*(a(6,1)*a(8,1)+a(2,1)*a(8,1)*a(4,1))...
            -5*a(1,1)*a(5,1)*a(10,1)-a(13,1)*(a(2,1)+a(4,1));
    f(:,26)=mmm{26}...
            -(a(1,1).^4)*(a(15,1)+3*a(9,1)*a(4,1)+a(6,1)*a(7,1))...
            -6*(a(1,1).^2)*(a(9,1)*a(3,1)+3*a(5,1)*a(3,1)*a(4,1)+a(2,1)*a(3,1)*a(7,1))...
            -4*a(1,1)*(a(6,1)*a(8,1)+3*a(2,1)*a(8,1)*a(4,1))...
            -a(10,1)*(a(5,1)+a(7,1));
    f(:,27)=mmm{27}...
            -(a(1,1).^3)*(a(15,1)+6*a(9,1)*a(4,1)+4*a(6,1)*a(7,1)+a(5,1)*a(11,1))...
            -a(1,1)*(3*a(9,1)*a(3,1)+18*a(5,1)*a(3,1)*a(4,1)+12*a(2,1)*a(3,1)*a(7,1))...
            -a(8,1)*(a(6,1)+6*a(2,1)*a(4,1)+a(11,1));
    f(:,28)=mmm{28}...
            -(a(1,1).^2)*(a(15,1)+10*a(9,1)*a(4,1)+10*a(6,1)*a(7,1)+5*a(5,1)*a(11,1)...
            +a(2,1)*a(14,1))...
            -a(3,1)*(a(9,1)+10*a(5,1)*a(4,1)+10*a(2,1)*a(7,1)+a(14,1));
    f(:,29)=mmm{29}...
            -a(1,1)*(a(15,1)+15*a(9,1)*a(4,1)+20*a(6,1)*a(7,1)+15*a(5,1)*a(11,1)...
            +6*a(2,1)*a(14,1));
end

f=f';



%subroutine to define the f vector for the optimal weighting matrix.
function f = EWoptw(mmm,ooo,zzz,n,neq,estim,SE)
f=zeros(n,neq);

f(:,1)=mmm{1}-ooo{1};
f(:,2)=mmm{2}-ooo{2};
f(:,3)=mmm{3}-ooo{3};
f(:,4)=mmm{4}-ooo{4};
f(:,5)=mmm{5}-ooo{5};

if estim>1
    f(:,6)=mmm{6}-ooo{6};
    f(:,7)=mmm{7}-ooo{7};
    f(:,8)=mmm{8}-ooo{8};
end

if estim>2
    f(:,9)=mmm{9}-ooo{9};
    f(:,10)=mmm{10}-ooo{10};
    f(:,11)=mmm{11}-ooo{11};
    f(:,12)=mmm{12}-ooo{12};
    f(:,13)=mmm{13}-ooo{13};
    f(:,14)=mmm{14}-ooo{14};
end

if estim>3
    f(:,15)=mmm{15}-ooo{15};
    f(:,16)=mmm{16}-ooo{16};
    f(:,17)=mmm{17}-ooo{17};
    f(:,18)=mmm{18}-ooo{18};
    f(:,19)=mmm{19}-ooo{19};
    f(:,20)=mmm{20}-ooo{20};
    f(:,21)=mmm{21}-ooo{21};
end

if estim>4
    f(:,22)=mmm{22}-ooo{22};
    f(:,23)=mmm{23}-ooo{23};
    f(:,24)=mmm{24}-ooo{24};
    f(:,25)=mmm{25}-ooo{25};
    f(:,26)=mmm{26}-ooo{26};
    f(:,27)=mmm{27}-ooo{27};
    f(:,28)=mmm{28}-ooo{28};
    f(:,29)=mmm{29}-ooo{29};
end

%this part makes the standard error adjustment.

if SE==0
    inezz    = zzz{1};
    zy_d     = zzz{2};
    zx_d     = zzz{3};
    eyx_z    = zzz{4};
    ey2_z    = zzz{5};
    ex2_z    = zzz{6};
    ey3_z    = zzz{7};
    ey2x_z   = zzz{8};
    eyx2_z   = zzz{9};
    ex3_z    = zzz{10};
    ey4_z    = zzz{11};
    ey3x_z   = zzz{12};
    ey2x2_z  = zzz{13};
    eyx3_z   = zzz{14};
    ex4_z    = zzz{15};
    ey5_z    = zzz{16};
    ey4x_z   = zzz{17};
    ey3x2_z  = zzz{18};
    ey2x3_z  = zzz{19};
    eyx4_z   = zzz{20};
    ex5_z    = zzz{21};
    ey6_z    = zzz{22};
    ey5x_z   = zzz{23};
    ey4x2_z  = zzz{24};
    ey3x3_z  = zzz{25};
    ey2x4_z  = zzz{26};
    eyx5_z   = zzz{27};
    ex6_z    = zzz{28};

    ei=zeros(n,neq);

    ei(:,4)=(-2*eyx_z'*inezz*(zy_d')-ey2_z'*inezz*(zx_d'))';
    ei(:,5)=(-ex2_z'*inezz*zy_d'-2*eyx_z'*inezz*zx_d')';

    if estim>1
        ei(:,6)=(-3*ey2x_z'*inezz*zy_d'-ey3_z'*inezz*zx_d')';
        ei(:,7)=(-2*eyx2_z'*inezz*zy_d'-2*ey2x_z'*inezz*zx_d')';
        ei(:,8)=(-ex3_z'*inezz*zy_d'-3*eyx2_z'*inezz*zx_d')';
    end
    
    if estim>2
        ei(:,9)=(-3*ex2_z'*inezz*zx_d')';
        ei(:,10)=(-3*ey2_z'*inezz*zy_d')';
        ei(:,11)=(-4*ey3x_z'*inezz*zy_d'-ey4_z'*inezz*zx_d')';
        ei(:,12)=(-3*ey2x2_z'*inezz*zy_d'-2*ey3x_z'*inezz*zx_d')';
        ei(:,13)=(-2*eyx3_z'*inezz*zy_d'-3*ey2x2_z'*inezz*zx_d')';
        ei(:,14)=(-ex4_z'*inezz*zy_d'-4*eyx3_z'*inezz*zx_d')';
    end

    if estim>3
        ei(:,15)=(-4*ey3_z'*inezz*zy_d')';
        ei(:,16)=(-4*ex3_z'*inezz*zx_d')';
        ei(:,17)=(-5*ey4x_z'*inezz*zy_d'-ey5_z'*inezz*zx_d')';
        ei(:,18)=(-4*ey3x2_z'*inezz*zy_d'-2*ey4x_z'*inezz*zx_d')';
        ei(:,19)=(-3*ey2x3_z'*inezz*zy_d'-3*ey3x2_z'*inezz*zx_d')';
        ei(:,20)=(-2*eyx4_z'*inezz*zy_d'-4*ey2x3_z'*inezz*zx_d')';
        ei(:,21)=(-ex5_z'*inezz*zy_d'-5*eyx4_z'*inezz*zx_d')';
    end

    if estim>4
        ei(:,22)=(-5*ey4_z'*inezz*zy_d')';
        ei(:,23)=(-5*ex4_z'*inezz*zx_d')';
        ei(:,24)=(-6*ey5x_z'*inezz*zy_d'-ey6_z'*inezz*zx_d')';
        ei(:,25)=(-5*ey4x2_z'*inezz*zy_d'-2*ey5x_z'*inezz*zx_d')';
        ei(:,26)=(-4*ey3x3_z'*inezz*zy_d'-3*ey4x2_z'*inezz*zx_d')';
        ei(:,27)=(-3*ey2x4_z'*inezz*zy_d'-4*ey3x3_z'*inezz*zx_d')';
        ei(:,28)=(-2*eyx5_z'*inezz*zy_d'-5*ey2x4_z'*inezz*zx_d')';
        ei(:,29)=(-ex6_z'*inezz*zy_d'-6*eyx5_z'*inezz*zx_d')';
    end

    f = f + ei;
end;


% -------------------------------------------------------
% Subroutine to compute squeezes.
%
%  This subroutine compares the values of the objective function at the
%  points c+s*dc and c+0.5*s*dc, with dc = the proposed change in the vector
%  of parameters, and with step length s initially at 1. s is halved until
%  minus the objective function stops declining.
function[s_c,s_iter] = EWsqueez(s_c,s_dc,emom,maxsqez,obj,estim,neq,w,ID)

s_c1=s_c - s_dc;  s_lm=1/2;  s_itr=1;
if (ID==3 || ID==4)
    s_f1 = EWIDcvec(emom,ID) - s_c1;
else
    s_f1 = EWdeff(s_c1,emom,estim,neq);
end

lc1 = s_f1'*w*s_f1;

while s_itr <= maxsqez

    s_c2=s_c-s_lm*s_dc;
    if (ID==3 || ID==4)
        s_f2 = EWIDcvec(emom,ID) - s_c2;
    else
        s_f2 = EWdeff(s_c2,emom,estim,neq);
    end
    lc2 = s_f2'*w*s_f2;
    clear s_f2;

    if lc1 <= lc2 && lc1 <= obj
       s_c = s_c1;  s_iter = s_itr-1;  
       return;
    else
       s_c1=s_c2;  s_lm=s_lm/2;  lc1=lc2;
       s_itr=s_itr+1;
    end

end
s_c = s_c2;  
s_iter = s_itr-1;


%subroutine to compute gradient matrix 1
function[dvdc] = grad1(a,neq)

dvdc1=zeros(neq,1);
dvdc2=zeros(neq,1);
dvdc3=zeros(neq,1);
dvdc4=zeros(neq,1);
dvdc5=zeros(neq,1);

dvdc1(1,1)=-2*a(1,1)*a(2,1);
dvdc1(2,1)=-a(2,1);
dvdc1(3,1)=0;
dvdc1(4,1)=-2*a(1,1)*a(5,1);
dvdc1(5,1)=-a(5,1);

dvdc2(1,1)=-(a(1,1).^2);
dvdc2(2,1)=-a(1,1);
dvdc2(3,1)=-1;
dvdc2(4,1)=0;
dvdc2(5,1)=0;

dvdc3(1,1)=-1;
dvdc3(2,1)=0;
dvdc3(3,1)=0;
dvdc3(4,1)=0;
dvdc3(5,1)=0;

dvdc4(1,1)=0;
dvdc4(2,1)=0;
dvdc4(3,1)=-1;
dvdc4(4,1)=0;
dvdc4(5,1)=0;

dvdc5(1,1)=0;
dvdc5(2,1)=0;
dvdc5(3,1)=0;
dvdc5(4,1)=-(a(1,1).^2);
dvdc5(5,1)=-a(1,1);

dvdc=[dvdc1,dvdc2,dvdc3,dvdc4,dvdc5];


%subroutine to compute gradient matrix 2
function[dvdc] = grad2(a,neq)

dvdc1=zeros(neq,1);
dvdc2=zeros(neq,1);
dvdc3=zeros(neq,1);
dvdc4=zeros(neq,1);
dvdc5=zeros(neq,1);
dvdc6=zeros(neq,1);

dvdc1(1,1)=-2*a(1,1)*a(2,1);
dvdc1(2,1)=-a(2,1);
dvdc1(3,1)=0;
dvdc1(4,1)=-2*a(1,1)*a(5,1);
dvdc1(5,1)=-a(5,1);
dvdc1(6,1)=-3*(a(1,1).^2)*a(6,1)-3*a(2,1)*a(3,1);
dvdc1(7,1)=-2*a(1,1)*(a(6,1)+a(2,1)*a(4,1));
dvdc1(8,1)=-a(6,1)-3*a(2,1)*a(4,1);

dvdc2(1,1)=-(a(1,1).^2);
dvdc2(2,1)=-a(1,1);
dvdc2(3,1)=-1;
dvdc2(4,1)=0;
dvdc2(5,1)=0;
dvdc2(6,1)=-3*a(1,1)*a(3,1);
dvdc2(7,1)=-(a(1,1).^2)*a(4,1)-a(3,1);
dvdc2(8,1)=-3*a(1,1)*a(4,1);

dvdc3(1,1)=-1;
dvdc3(2,1)=0;
dvdc3(3,1)=0;
dvdc3(4,1)=0;
dvdc3(5,1)=0;
dvdc3(6,1)=-3*a(1,1)*a(2,1);
dvdc3(7,1)=-(a(2,1)+a(4,1));
dvdc3(8,1)=0;

dvdc4(1,1)=0;
dvdc4(2,1)=0;
dvdc4(3,1)=-1;
dvdc4(4,1)=0;
dvdc4(5,1)=0;
dvdc4(6,1)=0;
dvdc4(7,1)=-(a(1,1).^2)*a(2,1)-a(3,1);
dvdc4(8,1)=-3*a(1,1)*a(2,1);

dvdc5(1,1)=0;
dvdc5(2,1)=0;
dvdc5(3,1)=0;
dvdc5(4,1)=-(a(1,1).^2);
dvdc5(5,1)=-a(1,1);
dvdc5(6,1)=0;
dvdc5(7,1)=0;
dvdc5(8,1)=0;

dvdc6(1,1)=0;
dvdc6(2,1)=0;
dvdc6(3,1)=0;
dvdc6(4,1)=0;
dvdc6(5,1)=0;
dvdc6(6,1)=-(a(1,1).^3);
dvdc6(7,1)=-(a(1,1).^2);
dvdc6(8,1)=-a(1,1);

dvdc=[dvdc1,dvdc2,dvdc3,dvdc4,dvdc5,dvdc6];


%subroutine to compute gradient matrix 3
function[dvdc] = grad3(a,neq)

dvdc1=zeros(neq,1);
dvdc2=zeros(neq,1);
dvdc3=zeros(neq,1);
dvdc4=zeros(neq,1);
dvdc5=zeros(neq,1);
dvdc6=zeros(neq,1);
dvdc7=zeros(neq,1);
dvdc8=zeros(neq,1);
dvdc9=zeros(neq,1);

dvdc1(1,1)=-2*a(1,1)*a(2,1);
dvdc1(2,1)=-a(2,1);
dvdc1(3,1)=0;
dvdc1(4,1)=-2*a(1,1)*a(5,1);
dvdc1(5,1)=-a(5,1);
dvdc1(6,1)=-3*(a(1,1).^2)*a(6,1)-3*a(2,1)*a(3,1);
dvdc1(7,1)=-2*a(1,1)*(a(6,1)+a(2,1)*a(4,1));
dvdc1(8,1)=-a(6,1)-3*a(2,1)*a(4,1);
dvdc1(9,1)=0;
dvdc1(10,1)=-3*(a(1,1).^2)*a(5,1);
dvdc1(11,1)=-4*(a(1,1).^3)*a(9,1)-12*a(1,1)*a(5,1)*a(3,1)...
            -4*a(2,1)*a(8,1);
dvdc1(12,1)=-3*(a(1,1).^2)*(a(9,1)+a(5,1)*a(4,1))...
            -3*a(5,1)*a(3,1);
dvdc1(13,1)=-2*a(1,1)*(a(9,1)+3*a(5,1)*a(4,1)+a(2,1)*a(7,1));
dvdc1(14,1)=-(a(9,1)+6*a(5,1)*a(4,1)+4*a(2,1)*a(7,1));

dvdc2(1,1)=-(a(1,1).^2);
dvdc2(2,1)=-a(1,1);
dvdc2(3,1)=-1;
dvdc2(4,1)=0;
dvdc2(5,1)=0;
dvdc2(6,1)=-3*a(1,1)*a(3,1);
dvdc2(7,1)=-(a(1,1).^2)*a(4,1)-a(3,1);
dvdc2(8,1)=-3*a(1,1)*a(4,1);
dvdc2(9,1)=0;
dvdc2(10,1)=0;
dvdc2(11,1)=-4*a(1,1)*a(8,1);
dvdc2(12,1)=-a(8,1);
dvdc2(13,1)=-(a(1,1).^2)*a(7,1);
dvdc2(14,1)=-4*a(1,1)*a(7,1);

dvdc3(1,1)=-1;
dvdc3(2,1)=0;
dvdc3(3,1)=0;
dvdc3(4,1)=0;
dvdc3(5,1)=0;
dvdc3(6,1)=-3*a(1,1)*a(2,1);
dvdc3(7,1)=-(a(2,1)+a(4,1));
dvdc3(8,1)=0;
dvdc3(9,1)=0;
dvdc3(10,1)=0;
dvdc3(11,1)=-6*(a(1,1).^2)*a(5,1);
dvdc3(12,1)=-3*a(1,1)*a(5,1);
dvdc3(13,1)=-(a(5,1)+a(7,1));
dvdc3(14,1)=0;

dvdc4(1,1)=0;
dvdc4(2,1)=0;
dvdc4(3,1)=-1;
dvdc4(4,1)=0;
dvdc4(5,1)=0;
dvdc4(6,1)=0;
dvdc4(7,1)=-(a(1,1).^2)*a(2,1)-a(3,1);
dvdc4(8,1)=-3*a(1,1)*a(2,1);
dvdc4(9,1)=0;
dvdc4(10,1)=0;
dvdc4(11,1)=0;
dvdc4(12,1)=-(a(1,1).^3)*a(5,1)-a(8,1);
dvdc4(13,1)=-3*(a(1,1).^2)*a(5,1);
dvdc4(14,1)=-6*a(1,1)*a(5,1);

dvdc5(1,1)=0;
dvdc5(2,1)=0;
dvdc5(3,1)=0;
dvdc5(4,1)=-(a(1,1).^2);
dvdc5(5,1)=-a(1,1);
dvdc5(6,1)=0;
dvdc5(7,1)=0;
dvdc5(8,1)=0;
dvdc5(9,1)=-1;
dvdc5(10,1)=-(a(1,1).^3);
dvdc5(11,1)=-6*(a(1,1).^2)*a(3,1);
dvdc5(12,1)=-(a(1,1).^3)*a(4,1)-3*a(1,1)*a(3,1);
dvdc5(13,1)=-3*(a(1,1).^2)*a(4,1)-a(3,1);
dvdc5(14,1)=-6*a(1,1)*a(4,1);

dvdc6(1,1)=0;
dvdc6(2,1)=0;
dvdc6(3,1)=0;
dvdc6(4,1)=0;
dvdc6(5,1)=0;
dvdc6(6,1)=-(a(1,1).^3);
dvdc6(7,1)=-(a(1,1).^2);
dvdc6(8,1)=-a(1,1);
dvdc6(9,1)=0;
dvdc6(10,1)=0;
dvdc6(11,1)=0;
dvdc6(12,1)=0;
dvdc6(13,1)=0;
dvdc6(14,1)=0;

dvdc7(1,1)=0;
dvdc7(2,1)=0;
dvdc7(3,1)=0;
dvdc7(4,1)=0;
dvdc7(5,1)=0;
dvdc7(6,1)=0;
dvdc7(7,1)=0;
dvdc7(8,1)=0;
dvdc7(9,1)=-1;
dvdc7(10,1)=0;
dvdc7(11,1)=0;
dvdc7(12,1)=0;
dvdc7(13,1)=-(a(1,1).^2)*a(2,1)-a(3,1);
dvdc7(14,1)=-4*a(1,1)*a(2,1);

dvdc8(1,1)=0;
dvdc8(2,1)=0;
dvdc8(3,1)=0;
dvdc8(4,1)=0;
dvdc8(5,1)=0;
dvdc8(6,1)=0;
dvdc8(7,1)=0;
dvdc8(8,1)=0;
dvdc8(9,1)=0;
dvdc8(10,1)=-1;
dvdc8(11,1)=-4*a(1,1)*a(2,1);
dvdc8(12,1)=-(a(2,1)+a(4,1));
dvdc8(13,1)=0;
dvdc8(14,1)=0;

dvdc9(1,1)=0;
dvdc9(2,1)=0;
dvdc9(3,1)=0;
dvdc9(4,1)=0;
dvdc9(5,1)=0;
dvdc9(6,1)=0;
dvdc9(7,1)=0;
dvdc9(8,1)=0;
dvdc9(9,1)=0;
dvdc9(10,1)=0;
dvdc9(11,1)=-(a(1,1).^4);
dvdc9(12,1)=-(a(1,1).^3);
dvdc9(13,1)=-(a(1,1).^2);
dvdc9(14,1)=-a(1,1);

dvdc=[dvdc1,dvdc2,dvdc3,dvdc4,dvdc5,dvdc6,dvdc7,dvdc8,dvdc9];


%subroutine to compute gradient matrix 4
function[dvdc] = grad4(a,neq)

dvdc1=zeros(neq,1);
dvdc2=zeros(neq,1);
dvdc3=zeros(neq,1);
dvdc4=zeros(neq,1);
dvdc5=zeros(neq,1);
dvdc6=zeros(neq,1);
dvdc7=zeros(neq,1);
dvdc8=zeros(neq,1);
dvdc9=zeros(neq,1);
dvdc10=zeros(neq,1);
dvdc11=zeros(neq,1);
dvdc12=zeros(neq,1);

dvdc1(1,1)=-2*a(1,1)*a(2,1);
dvdc1(2,1)=-a(2,1);
dvdc1(3,1)=0;
dvdc1(4,1)=-2*a(1,1)*a(5,1);
dvdc1(5,1)=-a(5,1);
dvdc1(6,1)=-3*(a(1,1).^2)*a(6,1)-3*a(2,1)*a(3,1);
dvdc1(7,1)=-2*a(1,1)*(a(6,1)+a(2,1)*a(4,1));
dvdc1(8,1)=-a(6,1)-3*a(2,1)*a(4,1);
dvdc1(9,1)=0;
dvdc1(10,1)=-3*(a(1,1).^2)*a(5,1);
dvdc1(11,1)=-4*(a(1,1).^3)*a(9,1)-12*a(1,1)*a(5,1)*a(3,1)...
            -4*a(2,1)*a(8,1);
dvdc1(12,1)=-3*(a(1,1).^2)*(a(9,1)+a(5,1)*a(4,1))...
            -3*a(5,1)*a(3,1);
dvdc1(13,1)=-2*a(1,1)*(a(9,1)+3*a(5,1)*a(4,1)+a(2,1)*a(7,1));
dvdc1(14,1)=-(a(9,1)+6*a(5,1)*a(4,1)+4*a(2,1)*a(7,1));
dvdc1(15,1)=-4*(a(1,1).^3)*a(6,1)-12*a(1,1)*a(2,1)*a(3,1);
dvdc1(16,1)=0;
dvdc1(17,1)=-5*(a(1,1).^4)*a(12,1)-30*(a(1,1).^2)*a(6,1)*a(3,1)...
            -20*a(1,1)*a(5,1)*a(8,1)-5*a(2,1)*a(10,1);
dvdc1(18,1)=-4*(a(1,1).^3)*(a(12,1)+a(6,1)*a(4,1))...
            -12*a(1,1)*(a(6,1)*a(3,1)+a(2,1)*a(3,1)*a(4,1))...
            -4*a(5,1)*a(8,1);
dvdc1(19,1)=-3*(a(1,1).^2)*(a(12,1)+3*a(6,1)*a(4,1)+a(5,1)*a(7,1))...
            -(3*a(6,1)*a(3,1)+9*a(2,1)*a(3,1)*a(4,1));
dvdc1(20,1)=-2*a(1,1)*(a(12,1)+6*a(6,1)*a(4,1)+4*a(5,1)*a(7,1)...
            +a(2,1)*a(11,1));
dvdc1(21,1)=-(a(12,1)+10*a(6,1)*a(4,1)+10*a(5,1)*a(7,1)...
            +5*a(2,1)*a(11,1));

dvdc2(1,1)=-(a(1,1).^2);
dvdc2(2,1)=-a(1,1);
dvdc2(3,1)=-1;
dvdc2(4,1)=0;
dvdc2(5,1)=0;
dvdc2(6,1)=-3*a(1,1)*a(3,1);
dvdc2(7,1)=-(a(1,1).^2)*a(4,1)-a(3,1);
dvdc2(8,1)=-3*a(1,1)*a(4,1);
dvdc2(9,1)=0;
dvdc2(10,1)=0;
dvdc2(11,1)=-4*a(1,1)*a(8,1);
dvdc2(12,1)=-a(8,1);
dvdc2(13,1)=-(a(1,1).^2)*a(7,1);
dvdc2(14,1)=-4*a(1,1)*a(7,1);
dvdc2(15,1)=-6*(a(1,1).^2)*a(3,1);
dvdc2(16,1)=-6*a(4,1);
dvdc2(17,1)=-5*a(1,1)*a(10,1);
dvdc2(18,1)=-6*(a(1,1).^2)*a(3,1)*a(4,1)-a(10,1);
dvdc2(19,1)=-9*a(1,1)*a(3,1)*a(4,1);
dvdc2(20,1)=-(a(1,1).^2)*a(11,1)-6*a(3,1)*a(4,1);
dvdc2(21,1)=-5*a(1,1)*a(11,1);

dvdc3(1,1)=-1;
dvdc3(2,1)=0;
dvdc3(3,1)=0;
dvdc3(4,1)=0;
dvdc3(5,1)=0;
dvdc3(6,1)=-3*a(1,1)*a(2,1);
dvdc3(7,1)=-(a(2,1)+a(4,1));
dvdc3(8,1)=0;
dvdc3(9,1)=0;
dvdc3(10,1)=0;
dvdc3(11,1)=-6*(a(1,1).^2)*a(5,1);
dvdc3(12,1)=-3*a(1,1)*a(5,1);
dvdc3(13,1)=-(a(5,1)+a(7,1));
dvdc3(14,1)=0;
dvdc3(15,1)=-6*(a(1,1).^2)*a(2,1);
dvdc3(16,1)=0;
dvdc3(17,1)=-10*(a(1,1).^3)*a(6,1);
dvdc3(18,1)=-6*(a(1,1).^2)*(a(6,1)+a(2,1));
dvdc3(19,1)=-a(1,1)*(3*a(6,1)+9*a(2,1)*a(4,1));
dvdc3(20,1)=-(a(6,1)+6*a(2,1)*a(4,1)+a(11,1));
dvdc3(21,1)=0;

dvdc4(1,1)=0;
dvdc4(2,1)=0;
dvdc4(3,1)=-1;
dvdc4(4,1)=0;
dvdc4(5,1)=0;
dvdc4(6,1)=0;
dvdc4(7,1)=-(a(1,1).^2)*a(2,1)-a(3,1);
dvdc4(8,1)=-3*a(1,1)*a(2,1);
dvdc4(9,1)=0;
dvdc4(10,1)=0;
dvdc4(11,1)=0;
dvdc4(12,1)=-(a(1,1).^3)*a(5,1)-a(8,1);
dvdc4(13,1)=-3*(a(1,1).^2)*a(5,1);
dvdc4(14,1)=-6*a(1,1)*a(5,1);
dvdc4(15,1)=0;
dvdc4(16,1)=-6*a(2,1);
dvdc4(17,1)=0;
dvdc4(18,1)=-(a(1,1).^4)*a(6,1)-6*(a(1,1).^2)*a(2,1)*a(3,1)-a(10,1);
dvdc4(19,1)=-3*(a(1,1).^3)*a(6,1)-9*a(1,1)*a(2,1)*a(3,1);
dvdc4(20,1)=-6*(a(1,1).^2)*a(6,1)-6*a(2,1)*a(3,1);
dvdc4(21,1)=-10*a(1,1)*a(6,1);

dvdc5(1,1)=0;
dvdc5(2,1)=0;
dvdc5(3,1)=0;
dvdc5(4,1)=-(a(1,1).^2);
dvdc5(5,1)=-a(1,1);
dvdc5(6,1)=0;
dvdc5(7,1)=0;
dvdc5(8,1)=0;
dvdc5(9,1)=-1;
dvdc5(10,1)=-(a(1,1).^3);
dvdc5(11,1)=-6*(a(1,1).^2)*a(3,1);
dvdc5(12,1)=-(a(1,1).^3)*a(4,1)-3*a(1,1)*a(3,1);
dvdc5(13,1)=-3*(a(1,1).^2)*a(4,1)-a(3,1);
dvdc5(14,1)=-6*a(1,1)*a(4,1);
dvdc5(15,1)=0;
dvdc5(16,1)=0;
dvdc5(17,1)=-10*(a(1,1).^2)*a(8,1);
dvdc5(18,1)=-4*a(1,1)*a(8,1);
dvdc5(19,1)=-(a(1,1).^3)*a(7,1)-a(8,1);
dvdc5(20,1)=-4*(a(1,1).^2)*a(7,1);
dvdc5(21,1)=-10*a(1,1)*a(7,1);

dvdc6(1,1)=0;
dvdc6(2,1)=0;
dvdc6(3,1)=0;
dvdc6(4,1)=0;
dvdc6(5,1)=0;
dvdc6(6,1)=-(a(1,1).^3);
dvdc6(7,1)=-(a(1,1).^2);
dvdc6(8,1)=-a(1,1);
dvdc6(9,1)=0;
dvdc6(10,1)=0;
dvdc6(11,1)=0;
dvdc6(12,1)=0;
dvdc6(13,1)=0;
dvdc6(14,1)=0;
dvdc6(15,1)=-(a(1,1).^4);
dvdc6(16,1)=-1;
dvdc6(17,1)=-10*(a(1,1).^3)*a(3,1);
dvdc6(18,1)=-(a(1,1).^4)*a(4,1)-6*(a(1,1).^2)*a(3,1);
dvdc6(19,1)=-3*(a(1,1).^3)*a(4,1)-3*a(1,1)*a(3,1);
dvdc6(20,1)=-6*(a(1,1).^2)*a(4,1)-a(3,1);
dvdc6(21,1)=-10*a(1,1)*a(4,1);

dvdc7(1,1)=0;
dvdc7(2,1)=0;
dvdc7(3,1)=0;
dvdc7(4,1)=0;
dvdc7(5,1)=0;
dvdc7(6,1)=0;
dvdc7(7,1)=0;
dvdc7(8,1)=0;
dvdc7(9,1)=-1;
dvdc7(10,1)=0;
dvdc7(11,1)=0;
dvdc7(12,1)=0;
dvdc7(13,1)=-(a(1,1).^2)*a(2,1)-a(3,1);
dvdc7(14,1)=-4*a(1,1)*a(2,1);
dvdc7(15,1)=0;
dvdc7(16,1)=0;
dvdc7(17,1)=0;
dvdc7(18,1)=0;
dvdc7(19,1)=-(a(1,1).^3)*a(5,1)-a(8,1);
dvdc7(20,1)=-4*(a(1,1).^2)*a(5,1);
dvdc7(21,1)=-10*a(1,1)*a(5,1);

dvdc8(1,1)=0;
dvdc8(2,1)=0;
dvdc8(3,1)=0;
dvdc8(4,1)=0;
dvdc8(5,1)=0;
dvdc8(6,1)=0;
dvdc8(7,1)=0;
dvdc8(8,1)=0;
dvdc8(9,1)=0;
dvdc8(10,1)=-1;
dvdc8(11,1)=-4*a(1,1)*a(2,1);
dvdc8(12,1)=-(a(2,1)+a(4,1));
dvdc8(13,1)=0;
dvdc8(14,1)=0;
dvdc8(15,1)=0;
dvdc8(16,1)=0;
dvdc8(17,1)=-10*(a(1,1).^2)*a(5,1);
dvdc8(18,1)=-4*a(1,1)*a(5,1);
dvdc8(19,1)=-(a(5,1)+a(7,1));
dvdc8(20,1)=0;
dvdc8(21,1)=0;

dvdc9(1,1)=0;
dvdc9(2,1)=0;
dvdc9(3,1)=0;
dvdc9(4,1)=0;
dvdc9(5,1)=0;
dvdc9(6,1)=0;
dvdc9(7,1)=0;
dvdc9(8,1)=0;
dvdc9(9,1)=0;
dvdc9(10,1)=0;
dvdc9(11,1)=-(a(1,1).^4);
dvdc9(12,1)=-(a(1,1).^3);
dvdc9(13,1)=-(a(1,1).^2);
dvdc9(14,1)=-a(1,1);
dvdc9(15,1)=0;
dvdc9(16,1)=0;
dvdc9(17,1)=0;
dvdc9(18,1)=0;
dvdc9(19,1)=0;
dvdc9(20,1)=0;
dvdc9(21,1)=0;

dvdc10(1,1)=0;
dvdc10(2,1)=0;
dvdc10(3,1)=0;
dvdc10(4,1)=0;
dvdc10(5,1)=0;
dvdc10(6,1)=0;
dvdc10(7,1)=0;
dvdc10(8,1)=0;
dvdc10(9,1)=0;
dvdc10(10,1)=0;
dvdc10(11,1)=0;
dvdc10(12,1)=0;
dvdc10(13,1)=0;
dvdc10(14,1)=0;
dvdc10(15,1)=-1;
dvdc10(16,1)=0;
dvdc10(17,1)=-5*a(1,1)*a(2,1);
dvdc10(18,1)=-(a(2,1)+a(4,1));
dvdc10(19,1)=0;
dvdc10(20,1)=0;
dvdc10(21,1)=0;

dvdc11(1,1)=0;
dvdc11(2,1)=0;
dvdc11(3,1)=0;
dvdc11(4,1)=0;
dvdc11(5,1)=0;
dvdc11(6,1)=0;
dvdc11(7,1)=0;
dvdc11(8,1)=0;
dvdc11(9,1)=0;
dvdc11(10,1)=0;
dvdc11(11,1)=0;
dvdc11(12,1)=0;
dvdc11(13,1)=0;
dvdc11(14,1)=0;
dvdc11(15,1)=0;
dvdc11(16,1)=-1;
dvdc11(17,1)=0;
dvdc11(18,1)=0;
dvdc11(19,1)=0;
dvdc11(20,1)=-(a(1,1).^2)*a(2,1)-a(3,1);
dvdc11(21,1)=-5*a(1,1)*a(2,1);

dvdc12(1,1)=0;
dvdc12(2,1)=0;
dvdc12(3,1)=0;
dvdc12(4,1)=0;
dvdc12(5,1)=0;
dvdc12(6,1)=0;
dvdc12(7,1)=0;
dvdc12(8,1)=0;
dvdc12(9,1)=0;
dvdc12(10,1)=0;
dvdc12(11,1)=0;
dvdc12(12,1)=0;
dvdc12(13,1)=0;
dvdc12(14,1)=0;
dvdc12(15,1)=0;
dvdc12(16,1)=0;
dvdc12(17,1)=-(a(1,1).^5);
dvdc12(18,1)=-(a(1,1).^4);
dvdc12(19,1)=-(a(1,1).^3);
dvdc12(20,1)=-(a(1,1).^2);
dvdc12(21,1)=-a(1,1);

dvdc=[dvdc1,dvdc2,dvdc3,dvdc4,dvdc5,dvdc6,dvdc7,dvdc8,dvdc9,dvdc10,dvdc11,dvdc12];


%subroutine to compute gradient matrix 5
function[dvdc] = grad5(a,neq)

dvdc1=zeros(neq,1);
dvdc2=zeros(neq,1);
dvdc3=zeros(neq,1);
dvdc4=zeros(neq,1);
dvdc5=zeros(neq,1);
dvdc6=zeros(neq,1);
dvdc7=zeros(neq,1);
dvdc8=zeros(neq,1);
dvdc9=zeros(neq,1);
dvdc10=zeros(neq,1);
dvdc11=zeros(neq,1);
dvdc12=zeros(neq,1);
dvdc13=zeros(neq,1);
dvdc14=zeros(neq,1);
dvdc15=zeros(neq,1);

dvdc1(1,1)=-2*a(1,1)*a(2,1);
dvdc1(2,1)=-a(2,1);
dvdc1(3,1)=0;
dvdc1(4,1)=-2*a(1,1)*a(5,1);
dvdc1(5,1)=-a(5,1);
dvdc1(6,1)=-3*(a(1,1).^2)*a(6,1)-3*a(2,1)*a(3,1);
dvdc1(7,1)=-2*a(1,1)*(a(6,1)+a(2,1)*a(4,1));
dvdc1(8,1)=-a(6,1)-3*a(2,1)*a(4,1);
dvdc1(9,1)=0;
dvdc1(10,1)=-3*(a(1,1).^2)*a(5,1);
dvdc1(11,1)=-4*(a(1,1).^3)*a(9,1)-12*a(1,1)*a(5,1)*a(3,1)...
            -4*a(2,1)*a(8,1);
dvdc1(12,1)=-3*(a(1,1).^2)*(a(9,1)+a(5,1)*a(4,1))...
            -3*a(5,1)*a(3,1);
dvdc1(13,1)=-2*a(1,1)*(a(9,1)+3*a(5,1)*a(4,1)+a(2,1)*a(7,1));
dvdc1(14,1)=-(a(9,1)+6*a(5,1)*a(4,1)+4*a(2,1)*a(7,1));
dvdc1(15,1)=-4*(a(1,1).^3)*a(6,1)-12*a(1,1)*a(2,1)*a(3,1);
dvdc1(16,1)=0;
dvdc1(17,1)=-5*(a(1,1).^4)*a(12,1)-30*(a(1,1).^2)*a(6,1)*a(3,1)...
            -20*a(1,1)*a(5,1)*a(8,1)-5*a(2,1)*a(10,1);
dvdc1(18,1)=-4*(a(1,1).^3)*(a(12,1)+a(6,1)*a(4,1))...
            -12*a(1,1)*(a(6,1)*a(3,1)+a(2,1)*a(3,1)*a(4,1))...
            -4*a(5,1)*a(8,1);
dvdc1(19,1)=-3*(a(1,1).^2)*(a(12,1)+3*a(6,1)*a(4,1)+a(5,1)*a(7,1))...
            -(3*a(6,1)*a(3,1)+9*a(2,1)*a(3,1)*a(4,1));
dvdc1(20,1)=-2*a(1,1)*(a(12,1)+6*a(6,1)*a(4,1)+4*a(5,1)*a(7,1)...
            +a(2,1)*a(11,1));
dvdc1(21,1)=-(a(12,1)+10*a(6,1)*a(4,1)+10*a(5,1)*a(7,1)...
            +5*a(2,1)*a(11,1));
dvdc1(22,1)=-5*(a(1,1).^4)*a(9,1)-30*(a(1,1).^2)*a(5,1)*a(3,1)...
            -20*a(1,1)*a(2,1)*a(8,1);
dvdc1(23,1)=0;
dvdc1(24,1)=-6*(a(1,1).^5)*a(15,1)-60*(a(1,1).^3)*a(9,1)*a(3,1)...
            -60*(a(1,1).^2)*a(6,1)*a(8,1)-30*a(1,1)*a(5,1)*a(10,1)...
            -6*a(2,1)*a(13,1);
dvdc1(25,1)=-5*(a(1,1).^4)*(a(15,1)+a(9,1)*a(4,1))...
            -30*(a(1,1).^2)*(a(9,1)*a(3,1)+a(5,1)*a(3,1)*a(4,1))...
            -20*a(1,1)*(a(6,1)*a(8,1)+a(2,1)*a(8,1)*a(4,1))...
            -5*a(5,1)*a(10,1);
dvdc1(26,1)=-4*(a(1,1).^3)*(a(15,1)+3*a(9,1)*a(4,1)+a(6,1)*a(7,1))...
            -12*a(1,1)*(a(9,1)*a(3,1)+3*a(5,1)*a(3,1)*a(4,1)+a(2,1)*a(3,1)*a(7,1))...
            -4*(a(6,1)*a(8,1)+3*a(2,1)*a(8,1)*a(4,1));
dvdc1(27,1)=-3*(a(1,1).^2)*(a(15,1)+6*a(9,1)*a(4,1)+4*a(6,1)*a(7,1)+a(5,1)*a(11,1))...
            -(3*a(9,1)*a(3,1)+18*a(5,1)*a(3,1)*a(4,1)+12*a(2,1)*a(3,1)*a(7,1));
dvdc1(28,1)=-2*a(1,1)*(a(15,1)+10*a(9,1)*a(4,1)+10*a(6,1)*a(7,1)+5*a(5,1)*a(11,1)...
            +a(2,1)*a(14,1));
dvdc1(29,1)=-(a(15,1)+15*a(9,1)*a(4,1)+20*a(6,1)*a(7,1)...
            +15*a(5,1)*a(11,1)+6*a(2,1)*a(14,1));

dvdc2(1,1)=-(a(1,1).^2);
dvdc2(2,1)=-a(1,1);
dvdc2(3,1)=-1;
dvdc2(4,1)=0;
dvdc2(5,1)=0;
dvdc2(6,1)=-3*a(1,1)*a(3,1);
dvdc2(7,1)=-(a(1,1).^2)*a(4,1)-a(3,1);
dvdc2(8,1)=-3*a(1,1)*a(4,1);
dvdc2(9,1)=0;
dvdc2(10,1)=0;
dvdc2(11,1)=-4*a(1,1)*a(8,1);
dvdc2(12,1)=-a(8,1);
dvdc2(13,1)=-(a(1,1).^2)*a(7,1);
dvdc2(14,1)=-4*a(1,1)*a(7,1);
dvdc2(15,1)=-6*(a(1,1).^2)*a(3,1);
dvdc2(16,1)=-6*a(4,1);
dvdc2(17,1)=-5*a(1,1)*a(10,1);
dvdc2(18,1)=-6*(a(1,1).^2)*a(3,1)*a(4,1)-a(10,1);
dvdc2(19,1)=-9*a(1,1)*a(3,1)*a(4,1);
dvdc2(20,1)=-(a(1,1).^2)*a(11,1)-6*a(3,1)*a(4,1);
dvdc2(21,1)=-5*a(1,1)*a(11,1);
dvdc2(22,1)=-10*(a(1,1).^2)*a(8,1);
dvdc2(23,1)=-10*a(7,1);
dvdc2(24,1)=-6*a(1,1)*a(13,1);
dvdc2(25,1)=-10*(a(1,1).^2)*a(8,1)*a(4,1)-a(13,1);
dvdc2(26,1)=-6*(a(1,1).^2)*a(3,1)*a(7,1)-12*a(1,1)*a(8,1)*a(4,1);
dvdc2(27,1)=-12*a(1,1)*a(3,1)*a(7,1)-6*a(8,1)*a(4,1);
dvdc2(28,1)=-(a(1,1).^2)*a(14,1)-10*a(3,1)*a(7,1);
dvdc2(29,1)=-6*a(1,1)*a(14,1);

dvdc3(1,1)=-1;
dvdc3(2,1)=0;
dvdc3(3,1)=0;
dvdc3(4,1)=0;
dvdc3(5,1)=0;
dvdc3(6,1)=-3*a(1,1)*a(2,1);
dvdc3(7,1)=-(a(2,1)+a(4,1));
dvdc3(8,1)=0;
dvdc3(9,1)=0;
dvdc3(10,1)=0;
dvdc3(11,1)=-6*(a(1,1).^2)*a(5,1);
dvdc3(12,1)=-3*a(1,1)*a(5,1);
dvdc3(13,1)=-(a(5,1)+a(7,1));
dvdc3(14,1)=0;
dvdc3(15,1)=-6*(a(1,1).^2)*a(2,1);
dvdc3(16,1)=0;
dvdc3(17,1)=-10*(a(1,1).^3)*a(6,1);
dvdc3(18,1)=-6*(a(1,1).^2)*(a(6,1)+a(2,1));
dvdc3(19,1)=-a(1,1)*(3*a(6,1)+9*a(2,1)*a(4,1));
dvdc3(20,1)=-(a(6,1)+6*a(2,1)*a(4,1)+a(11,1));
dvdc3(21,1)=0;
dvdc3(22,1)=-10*(a(1,1).^3)*a(5,1);
dvdc3(23,1)=0;
dvdc3(24,1)=-15*(a(1,1).^4)*a(9,1);
dvdc3(25,1)=-10*(a(1,1).^3)*(a(9,1)+a(5,1)*a(4,1));
dvdc3(26,1)=-6*(a(1,1).^2)*(a(9,1)+3*a(5,1)*a(4,1)+a(2,1)*a(7,1));
dvdc3(27,1)=-a(1,1)*(3*a(9,1)+18*a(5,1)*a(4,1)+12*a(2,1)*a(7,1));
dvdc3(28,1)=-(a(9,1)+10*a(5,1)*a(4,1)+10*a(2,1)*a(7,1)+a(14,1));
dvdc3(29,1)=0;

dvdc4(1,1)=0;
dvdc4(2,1)=0;
dvdc4(3,1)=-1;
dvdc4(4,1)=0;
dvdc4(5,1)=0;
dvdc4(6,1)=0;
dvdc4(7,1)=-(a(1,1).^2)*a(2,1)-a(3,1);
dvdc4(8,1)=-3*a(1,1)*a(2,1);
dvdc4(9,1)=0;
dvdc4(10,1)=0;
dvdc4(11,1)=0;
dvdc4(12,1)=-(a(1,1).^3)*a(5,1)-a(8,1);
dvdc4(13,1)=-3*(a(1,1).^2)*a(5,1);
dvdc4(14,1)=-6*a(1,1)*a(5,1);
dvdc4(15,1)=0;
dvdc4(16,1)=-6*a(2,1);
dvdc4(17,1)=0;
dvdc4(18,1)=-(a(1,1).^4)*a(6,1)-6*(a(1,1).^2)*a(2,1)*a(3,1)-a(10,1);
dvdc4(19,1)=-3*(a(1,1).^3)*a(6,1)-9*a(1,1)*a(2,1)*a(3,1);
dvdc4(20,1)=-6*(a(1,1).^2)*a(6,1)-6*a(2,1)*a(3,1);
dvdc4(21,1)=-10*a(1,1)*a(6,1);
dvdc4(22,1)=0;
dvdc4(23,1)=-10*a(5,1);
dvdc4(24,1)=0;
dvdc4(25,1)=-(a(1,1).^5)*a(9,1)-10*(a(1,1).^3)*a(5,1)*a(3,1)...
            -10*(a(1,1).^2)*a(2,1)*a(8,1)-a(13,1);
dvdc4(26,1)=-3*(a(1,1).^4)*a(9,1)-18*(a(1,1).^2)*a(5,1)*a(3,1)...
            -12*a(1,1)*a(2,1)*a(8,1);
dvdc4(27,1)=-6*(a(1,1).^3)*a(9,1)-18*a(1,1)*a(5,1)*a(3,1)...
            -6*a(8,1)*a(2,1);
dvdc4(28,1)=-10*(a(1,1).^2)*a(9,1)-10*a(3,1)*a(5,1);
dvdc4(29,1)=-15*a(1,1)*a(9,1);

dvdc5(1,1)=0;
dvdc5(2,1)=0;
dvdc5(3,1)=0;
dvdc5(4,1)=-(a(1,1).^2);
dvdc5(5,1)=-a(1,1);
dvdc5(6,1)=0;
dvdc5(7,1)=0;
dvdc5(8,1)=0;
dvdc5(9,1)=-1;
dvdc5(10,1)=-(a(1,1).^3);
dvdc5(11,1)=-6*(a(1,1).^2)*a(3,1);
dvdc5(12,1)=-(a(1,1).^3)*a(4,1)-3*a(1,1)*a(3,1);
dvdc5(13,1)=-3*(a(1,1).^2)*a(4,1)-a(3,1);
dvdc5(14,1)=-6*a(1,1)*a(4,1);
dvdc5(15,1)=0;
dvdc5(16,1)=0;
dvdc5(17,1)=-10*(a(1,1).^2)*a(8,1);
dvdc5(18,1)=-4*a(1,1)*a(8,1);
dvdc5(19,1)=-(a(1,1).^3)*a(7,1)-a(8,1);
dvdc5(20,1)=-4*(a(1,1).^2)*a(7,1);
dvdc5(21,1)=-10*a(1,1)*a(7,1);
dvdc5(22,1)=-10*(a(1,1).^3)*a(3,1);
dvdc5(23,1)=-10*a(4,1);
dvdc5(24,1)=-15*(a(1,1).^2)*a(10,1);
dvdc5(25,1)=-10*(a(1,1).^3)*a(3,1)*a(4,1)-5*a(1,1)*a(10,1);
dvdc5(26,1)=-18*(a(1,1).^2)*a(3,1)*a(4,1)-a(10,1);
dvdc5(27,1)=-(a(1,1).^3)*a(11,1)-18*a(1,1)*a(3,1)*a(4,1);
dvdc5(28,1)=-5*(a(1,1).^2)*a(11,1)-10*a(3,1)*a(4,1);
dvdc5(29,1)=-15*a(1,1)*a(11,1);

dvdc6(1,1)=0;
dvdc6(2,1)=0;
dvdc6(3,1)=0;
dvdc6(4,1)=0;
dvdc6(5,1)=0;
dvdc6(6,1)=-(a(1,1).^3);
dvdc6(7,1)=-(a(1,1).^2);
dvdc6(8,1)=-a(1,1);
dvdc6(9,1)=0;
dvdc6(10,1)=0;
dvdc6(11,1)=0;
dvdc6(12,1)=0;
dvdc6(13,1)=0;
dvdc6(14,1)=0;
dvdc6(15,1)=-(a(1,1).^4);
dvdc6(16,1)=-1;
dvdc6(17,1)=-10*(a(1,1).^3)*a(3,1);
dvdc6(18,1)=-(a(1,1).^4)*a(4,1)-6*(a(1,1).^2)*a(3,1);
dvdc6(19,1)=-3*(a(1,1).^3)*a(4,1)-3*a(1,1)*a(3,1);
dvdc6(20,1)=-6*(a(1,1).^2)*a(4,1)-a(3,1);
dvdc6(21,1)=-10*a(1,1)*a(4,1);
dvdc6(22,1)=0;
dvdc6(23,1)=0;
dvdc6(24,1)=-20*(a(1,1).^3)*a(8,1);
dvdc6(25,1)=-10*(a(1,1).^2)*a(8,1);
dvdc6(26,1)=-(a(1,1).^4)*a(7,1)-4*a(1,1)*a(8,1);
dvdc6(27,1)=-4*(a(1,1).^3)*a(7,1)-a(8,1);
dvdc6(28,1)=-10*(a(1,1).^2)*a(7,1);
dvdc6(29,1)=-20*a(1,1)*a(7,1);

dvdc7(1,1)=0;
dvdc7(2,1)=0;
dvdc7(3,1)=0;
dvdc7(4,1)=0;
dvdc7(5,1)=0;
dvdc7(6,1)=0;
dvdc7(7,1)=0;
dvdc7(8,1)=0;
dvdc7(9,1)=-1;
dvdc7(10,1)=0;
dvdc7(11,1)=0;
dvdc7(12,1)=0;
dvdc7(13,1)=-(a(1,1).^2)*a(2,1)-a(3,1);
dvdc7(14,1)=-4*a(1,1)*a(2,1);
dvdc7(15,1)=0;
dvdc7(16,1)=0;
dvdc7(17,1)=0;
dvdc7(18,1)=0;
dvdc7(19,1)=-(a(1,1).^3)*a(5,1)-a(8,1);
dvdc7(20,1)=-4*(a(1,1).^2)*a(5,1);
dvdc7(21,1)=-10*a(1,1)*a(5,1);
dvdc7(22,1)=0;
dvdc7(23,1)=-10*a(2,1);
dvdc7(24,1)=0;
dvdc7(25,1)=0;
dvdc7(26,1)=-(a(1,1).^4)*a(6,1)-6*(a(1,1).^2)*a(2,1)*a(3,1)-a(10,1);
dvdc7(27,1)=-4*(a(1,1).^3)*a(6,1)-12*a(1,1)*a(2,1)*a(3,1);
dvdc7(28,1)=-10*(a(1,1).^2)*a(6,1)-10*a(3,1)*a(2,1);
dvdc7(29,1)=-20*a(1,1)*a(6,1);

dvdc8(1,1)=0;
dvdc8(2,1)=0;
dvdc8(3,1)=0;
dvdc8(4,1)=0;
dvdc8(5,1)=0;
dvdc8(6,1)=0;
dvdc8(7,1)=0;
dvdc8(8,1)=0;
dvdc8(9,1)=0;
dvdc8(10,1)=-1;
dvdc8(11,1)=-4*a(1,1)*a(2,1);
dvdc8(12,1)=-(a(2,1)+a(4,1));
dvdc8(13,1)=0;
dvdc8(14,1)=0;
dvdc8(15,1)=0;
dvdc8(16,1)=0;
dvdc8(17,1)=-10*(a(1,1).^2)*a(5,1);
dvdc8(18,1)=-4*a(1,1)*a(5,1);
dvdc8(19,1)=-(a(5,1)+a(7,1));
dvdc8(20,1)=0;
dvdc8(21,1)=0;
dvdc8(22,1)=-10*(a(1,1).^2)*a(2,1);
dvdc8(23,1)=0;
dvdc8(24,1)=-20*(a(1,1).^3)*a(6,1);
dvdc8(25,1)=-10*(a(1,1).^2)*(a(6,1)+a(2,1)*a(4,1));
dvdc8(26,1)=-4*a(1,1)*(a(6,1)+3*a(2,1)*a(4,1));
dvdc8(27,1)=-(a(6,1)+6*a(2,1)*a(4,1)+a(11,1));
dvdc8(28,1)=0;
dvdc8(29,1)=0;

dvdc9(1,1)=0;
dvdc9(2,1)=0;
dvdc9(3,1)=0;
dvdc9(4,1)=0;
dvdc9(5,1)=0;
dvdc9(6,1)=0;
dvdc9(7,1)=0;
dvdc9(8,1)=0;
dvdc9(9,1)=0;
dvdc9(10,1)=0;
dvdc9(11,1)=-(a(1,1).^4);
dvdc9(12,1)=-(a(1,1).^3);
dvdc9(13,1)=-(a(1,1).^2);
dvdc9(14,1)=-a(1,1);
dvdc9(15,1)=0;
dvdc9(16,1)=0;
dvdc9(17,1)=0;
dvdc9(18,1)=0;
dvdc9(19,1)=0;
dvdc9(20,1)=0;
dvdc9(21,1)=0;
dvdc9(22,1)=-(a(1,1).^5);
dvdc9(23,1)=-1;
dvdc9(24,1)=-15*(a(1,1).^4)*a(3,1);
dvdc9(25,1)=-(a(1,1).^5)*a(4,1)-10*(a(1,1).^3)*a(3,1);
dvdc9(26,1)=-3*(a(1,1).^4)*a(4,1)-6*(a(1,1).^2)*a(3,1);
dvdc9(27,1)=-6*(a(1,1).^3)*a(4,1)-3*a(1,1)*a(3,1);
dvdc9(28,1)=-10*(a(1,1).^2)*a(4,1)-a(3,1);
dvdc9(29,1)=-15*a(1,1)*a(4,1);

dvdc10(1,1)=0;
dvdc10(2,1)=0;
dvdc10(3,1)=0;
dvdc10(4,1)=0;
dvdc10(5,1)=0;
dvdc10(6,1)=0;
dvdc10(7,1)=0;
dvdc10(8,1)=0;
dvdc10(9,1)=0;
dvdc10(10,1)=0;
dvdc10(11,1)=0;
dvdc10(12,1)=0;
dvdc10(13,1)=0;
dvdc10(14,1)=0;
dvdc10(15,1)=-1;
dvdc10(16,1)=0;
dvdc10(17,1)=-5*a(1,1)*a(2,1);
dvdc10(18,1)=-(a(2,1)+a(4,1));
dvdc10(19,1)=0;
dvdc10(20,1)=0;
dvdc10(21,1)=0;
dvdc10(22,1)=0;
dvdc10(23,1)=0;
dvdc10(24,1)=-15*(a(1,1).^2)*a(5,1);
dvdc10(25,1)=-5*a(1,1)*a(5,1);
dvdc10(26,1)=-(a(5,1)+a(7,1));
dvdc10(27,1)=0;
dvdc10(28,1)=0;
dvdc10(29,1)=0;

dvdc11(1,1)=0;
dvdc11(2,1)=0;
dvdc11(3,1)=0;
dvdc11(4,1)=0;
dvdc11(5,1)=0;
dvdc11(6,1)=0;
dvdc11(7,1)=0;
dvdc11(8,1)=0;
dvdc11(9,1)=0;
dvdc11(10,1)=0;
dvdc11(11,1)=0;
dvdc11(12,1)=0;
dvdc11(13,1)=0;
dvdc11(14,1)=0;
dvdc11(15,1)=0;
dvdc11(16,1)=-1;
dvdc11(17,1)=0;
dvdc11(18,1)=0;
dvdc11(19,1)=0;
dvdc11(20,1)=-(a(1,1).^2)*a(2,1)-a(3,1);
dvdc11(21,1)=-5*a(1,1)*a(2,1);
dvdc11(22,1)=0;
dvdc11(23,1)=0;
dvdc11(24,1)=0;
dvdc11(25,1)=0;
dvdc11(26,1)=0;
dvdc11(27,1)=-(a(1,1).^3)*a(5,1)-a(8,1);
dvdc11(28,1)=-5*(a(1,1).^2)*a(5,1);
dvdc11(29,1)=-15*a(1,1)*a(5,1);

dvdc12(1,1)=0;
dvdc12(2,1)=0;
dvdc12(3,1)=0;
dvdc12(4,1)=0;
dvdc12(5,1)=0;
dvdc12(6,1)=0;
dvdc12(7,1)=0;
dvdc12(8,1)=0;
dvdc12(9,1)=0;
dvdc12(10,1)=0;
dvdc12(11,1)=0;
dvdc12(12,1)=0;
dvdc12(13,1)=0;
dvdc12(14,1)=0;
dvdc12(15,1)=0;
dvdc12(16,1)=0;
dvdc12(17,1)=-(a(1,1).^5);
dvdc12(18,1)=-(a(1,1).^4);
dvdc12(19,1)=-(a(1,1).^3);
dvdc12(20,1)=-(a(1,1).^2);
dvdc12(21,1)=-a(1,1);
dvdc12(22,1)=0;
dvdc12(23,1)=0;
dvdc12(24,1)=0;
dvdc12(25,1)=0;
dvdc12(26,1)=0;
dvdc12(27,1)=0;
dvdc12(28,1)=0;
dvdc12(29,1)=0;

dvdc13(1,1)=0;
dvdc13(2,1)=0;
dvdc13(3,1)=0;
dvdc13(4,1)=0;
dvdc13(5,1)=0;
dvdc13(6,1)=0;
dvdc13(7,1)=0;
dvdc13(8,1)=0;
dvdc13(9,1)=0;
dvdc13(10,1)=0;
dvdc13(11,1)=0;
dvdc13(12,1)=0;
dvdc13(13,1)=0;
dvdc13(14,1)=0;
dvdc13(15,1)=0;
dvdc13(16,1)=0;
dvdc13(17,1)=0;
dvdc13(18,1)=0;
dvdc13(19,1)=0;
dvdc13(20,1)=0;
dvdc13(21,1)=0;
dvdc13(22,1)=-1;
dvdc13(23,1)=0;
dvdc13(24,1)=-6*a(1,1)*a(2,1);
dvdc13(25,1)=-(a(2,1)+a(4,1));
dvdc13(26,1)=0;
dvdc13(27,1)=0;
dvdc13(28,1)=0;
dvdc13(29,1)=0;

dvdc14(1,1)=0;
dvdc14(2,1)=0;
dvdc14(3,1)=0;
dvdc14(4,1)=0;
dvdc14(5,1)=0;
dvdc14(6,1)=0;
dvdc14(7,1)=0;
dvdc14(8,1)=0;
dvdc14(9,1)=0;
dvdc14(10,1)=0;
dvdc14(11,1)=0;
dvdc14(12,1)=0;
dvdc14(13,1)=0;
dvdc14(14,1)=0;
dvdc14(15,1)=0;
dvdc14(16,1)=0;
dvdc14(17,1)=0;
dvdc14(18,1)=0;
dvdc14(19,1)=0;
dvdc14(20,1)=0;
dvdc14(21,1)=0;
dvdc14(22,1)=0;
dvdc14(23,1)=-1;
dvdc14(24,1)=0;
dvdc14(25,1)=0;
dvdc14(26,1)=0;
dvdc14(27,1)=0;
dvdc14(28,1)=-(a(1,1).^2)*a(2,1)-a(3,1);
dvdc14(29,1)=-6*a(1,1)*a(2,1);


dvdc15(1,1)=0;
dvdc15(2,1)=0;
dvdc15(3,1)=0;
dvdc15(4,1)=0;
dvdc15(5,1)=0;
dvdc15(6,1)=0;
dvdc15(7,1)=0;
dvdc15(8,1)=0;
dvdc15(9,1)=0;
dvdc15(10,1)=0;
dvdc15(11,1)=0;
dvdc15(12,1)=0;
dvdc15(13,1)=0;
dvdc15(14,1)=0;
dvdc15(15,1)=0;
dvdc15(16,1)=0;
dvdc15(17,1)=0;
dvdc15(18,1)=0;
dvdc15(19,1)=0;
dvdc15(20,1)=0;
dvdc15(21,1)=0;
dvdc15(22,1)=0;
dvdc15(23,1)=0;
dvdc15(24,1)=-(a(1,1).^6);
dvdc15(25,1)=-(a(1,1).^5);
dvdc15(26,1)=-(a(1,1).^4);
dvdc15(27,1)=-(a(1,1).^3);
dvdc15(28,1)=-(a(1,1).^2);
dvdc15(29,1)=-a(1,1);

dvdc=[dvdc1,dvdc2,dvdc3,dvdc4,dvdc5,dvdc6,dvdc7,dvdc8,dvdc9,dvdc10,dvdc11,dvdc12,dvdc13,dvdc14,dvdc15];
