function [bX, bZ, seX, seZ, stats] = XTEWreg(y, x, z, panvr, tmvr, meth, bXint, SE)
% XTEWreg - linear regression with one mismeasured regressor, using the
%           Erickson Whited [2000 JPE] method, and the minimum distance for
%           unbalanced panels method described in Erickson Whited [2011 RFS]
%
% y     - dependant variable - a [N,1] vector
%
% x     - independant mismeasured regressor - a [N,1] vector
%
% z     - independant perfectly measured regressors - a [N,NZ] matrix
%         note that Z must include an intercept (a vecotr of ones)
%
% panvr - panel variable - a [N,1] vector of id variable, which identifies
%         each individual in the panel (such as GVKEY, CUSIP, SSN, etc)
%
% tmvr  - time variable - a [N,1] vector of time id, such as year, month,
%         or any other counter of different time periods in the panel
%
% meth  - the method to use, one of {'GMM3', 'GMM4', 'GMM5', 'GMM6', 'GMM7'}
%       - Default: 'GMM3'
%
% bXint - initial values to use in GMM optimization for bX (only if meth is not 'GMM3')
%         a [M,1] vector of initial values.
%         This can be a list of values, and all values are ok except zero.
%         If a zero is found in the list, it will be replaced by the
%         defualt value. So it is advisable to have zero always be a member
%         of the list. The initial value yielding the lowest objective function 
%         will be used.
%       - Default: E(y_d^2*x_d)/E(y_d*x_d^2)
%
% SE    - Should we account for SE as if Ex and Ey are known?
%       - This will disable the SE adjustment when finding the optimal
%         weighting matrix for the GMM step
%       - Default: 0 (No)
%
%
% Output:
% bX    - scalar coefficient on the mismeasured regressor
%
% bZ    - [NZ,1] coefficients on the perfectly measured regressors
%
% seX   - scalar standard error on the mismeasured regressor coeff
%
% seZ   - [NZ,1] SE on the perfectly measured regressors coeff
%
% stats - a structure containing all returned statistics, including:
%       - cross{} - a cell array of length equal to number of time period in the panel. 
%                   each cell contains cross-sectional estimation results for a given 
%                   time period. each cell has the same form as EWreg stats
%                   structure, with 1 extra field, dt, which captures the
%                   time period used.
%       - bX      - scalar coefficient on the mismeasured regressor
%       - bZ      - [NZ,1] coefficients on the perfectly measured regressors
%       - seX     - scalar standard error on the mismeasured regressor coeff
%       - seZ     - [NZ,1] SE on the perfectly measured regressors coeff
%       - rho     - the R^2 value for the "true" regression
%       - tau     - the R^2 value for the mismeasured regression
%       - SErho   - standard error for rho^2
%       - SEtau   - standard error for tau^2
%       - cmdtestAAA - where AAA in {X,Z,Rho,Tau} - the chi squared test
%                      statistic on the minimum distance estimator
%       - cmdvalAAA  - where AAA in {X,Z,Rho,Tau} - the p-value on the 
%                      chi squared test statistic


if nargin<5
    error('XTEWreg:nargin', 'Not enough arguments, review function documentation')
end

n =size(y,1);
nz=size(z,2);

if nz==0 || sum(sum(z==1)==n)==0
    error('EWreg:const', 'Z must contain a "1" column')
end

if nargin==5
    meth='GMM3';
    bXint=0;
    SE=0;
elseif nargin==6
    bXint=0;
    SE=0;
elseif nargin==7
    SE=0;
end

periods = unique(sort(tmvr));
n_per   = size(periods,1);

pans    = unique(sort(panvr));
n_panel = size(pans,1);

r_csave = zeros(1,n_per);
r_isave = zeros(n_panel,n_per);
t_csave = zeros(1,n_per);
t_isave = zeros(n_panel,n_per);
b_csave = zeros(1,n_per);
b_isave = zeros(n_panel,n_per);

z_csave = zeros(nz,n_per);
z_isave = zeros(n_panel,nz,n_per);

touse = cell(1,n_per);
n_pan = zeros(1,n_per);

for t = 1:n_per

    fprintf('Now at period %4d of %4d : %5d\n',[t,n_per,periods(t)]);

    touse{t} = (tmvr==periods(t));
    [~,~,~,~,stats.cross{t}] = EWreg(y(touse{t}), x(touse{t}), z(touse{t},:), meth, bXint, SE);
    stats.cross{t}.dt = periods(t);
    
    n_pan(t) = sum(touse{t});
    [~,pos]  = ismember(pans, panvr(touse{t}));
    pos=pos(pos~=0);

    r_csave(1,t) = stats.cross{t}.rho;
    t_csave(1,t) = stats.cross{t}.tau;
    b_csave(1,t) = stats.cross{t}.bX;
    r_isave(pos,t) = stats.cross{t}.phirho;
    t_isave(pos,t) = stats.cross{t}.phitau;
    b_isave(pos,t) = stats.cross{t}.inflncX;

    z_csave(:,t) = stats.cross{t}.bZ;
    z_isave(pos,:,t) = stats.cross{t}.inflncZ;
end

%sharefrac hold nc/n1 as defined in eq A4 of EW2011 RFS
sharefrac = zeros(n_per,n_per);
for t1=1:n_per
    for t2=1:n_per
        % is the entity in the panel in both time periods?
        pan1 = panvr(touse{t1});
        pan2 = panvr(touse{t2});
        sharefrac(t1,t2) = sum(ismember(pan1,pan2))/n_pan(t1);
    end
end

[stats.rho, stats.SErho, stats.cmdtestRho, stats.cmdvalRho] = cmd(r_csave, r_isave, n_per, sharefrac);
[stats.tau, stats.SEtau, stats.cmdtestTau, stats.cmdvalTau] = cmd(t_csave, t_isave, n_per, sharefrac);
[stats.bX, stats.seX, stats.cmdtestX, stats.cmdvalX] = cmd(b_csave, b_isave, n_per, sharefrac);
for i=1:nz
    [stats.bZ(i,1), stats.seZ(i,1), stats.cmdtestZ(i,1), stats.cmdvalZ(i,1)] = cmd(z_csave(i,:), squeeze(z_isave(:,i,:)), n_per, sharefrac);
end

bX=stats.bX;
bZ=stats.bZ;
seX=stats.seX;
seZ=stats.seZ;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Helper Functions
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% subroutine to compute classical minimum distance estimator
% csave [1,S] - coefficients to be merged
% isave [N,S] - influence functions of coefficients
function [theta, stderr, cmdtest, cmdval] = cmd(csave, isave, n_period, sharefrac)
ninfl = isave;
ninfl(isave~=0)=1;
divideby = ninfl'*ninfl;
w = inv(isave'*isave.*sharefrac.*sharefrac'./(divideby.^2));
g=ones(n_period,1);
theta=inv(g'*w*g)*g'*w*csave';
f=csave'-theta;
stderr=sqrt(inv(g'*w*g)        );
cmdtest=        f'*w*f;
cmdval = 1-chi2cdf(cmdtest,n_period-1);

